# dice_similarity_coefficient_v011.py
#
# In this first version, we need two VMRs, where the FMR is saved as VMR: the FMR should be coregistered to VMR, create a VTC and then save as VMR by first clicking
# "Show VTC File" in 3D Volume Tools, then File > Save as Secondary VMR.
# Script assumes number of FMR voxels will be larger than VMR, hence will subtract VMR from FMR; if the Dice coefficient is negative,
# change subtraction order by commenting (#) line 54 and uncommenting line 53
# In second version, the FMR will be mapped to the VMR in the script.
#
# Usage: load script in BrainVoyager Python Script Editor, click 'Run'. Files will be closed when script is finished. 
# Dice similarity coefficient will be written to the BrainVoyager Log tab.
#
# Calculation |A ∩ B| / 1/2 (|A| + |B|) = 2|A ∩ B|  / |A| + |B| 
# or in terms of vector operations over binary vectors a and b: 
# 2| \mathbf{a} \cdot \mathbf{b} |  / |mathbf{a}|^{2} + |mathbf{b}|^{2}
# where 
# |A ∩ B| is the sum over all voxels that are marked in both images, ∩ is logical AND
# |A| is the size of all marked voxels in image A
# |B| is the size of all marked voxels in image B
# https://en.wikipedia.org/wiki/S%C3%B8rensen%E2%80%93Dice_coefficient
# Please note that the Dice similarity coefficient has many alternatives which might or might not be more appropriate.
#
# HB 230320 
# fix in calculation; also threshold for both images to 100; added check for comparing size of images HB 070420

import numpy as np
import bv

thresholdVMR = 100 # exclusive
thresholdFMR = 100

BrainVoyager.PrintToLog("\nScript to calculate the Dice similarity coefficient (or Sørensen–Dice coefficient)")
# open VMR, read data
vmrfilename = BrainVoyager.BrowseFile("Please select an VMR file", "*.vmr")
docvmr = BrainVoyager.OpenDocument(vmrfilename)
volumeDataVMR = docvmr.GetVolumeDataAsDoubleList()
t1Data1D = np.array(volumeDataVMR)                       # convert list (or tuple) to numpy array
BrainVoyager.PrintToLog('First file:\n' + vmrfilename)

# open FMR, read data
fmrfilename = BrainVoyager.BrowseFile("Please select a functional-in-anatomical file", "*.vmr")
docfmr = BrainVoyager.OpenDocument(fmrfilename)
volumeDataFMR = docfmr.GetVolumeDataAsDoubleList()
t2Data1D = np.array(volumeDataFMR)                       # convert list (or tuple) to numpy array
BrainVoyager.PrintToLog('Second file:\n' + fmrfilename)

# segment into binary images 
marked = 1
t1Data1D[t1Data1D <= thresholdVMR] = 0 # first this one, otherwise everything will be black
t1Data1D[t1Data1D > thresholdVMR] = marked # segment 
t2Data1D[t2Data1D <= thresholdFMR] = 0 
t2Data1D[t2Data1D > thresholdFMR] = marked 

 
# get size (= number of voxels): take sum of all marked voxels without background
sizeVMR = docvmr.DimX * docvmr.DimY * docvmr.DimZ
sizeFMR = docfmr.DimX * docfmr.DimY * docfmr.DimZ

if (sizeVMR != sizeFMR):
    BrainVoyager.PrintToLog("Images should have similar size")
else:
    sizeMarkedVMR = np.sum(t1Data1D)
    BrainVoyager.PrintToLog("Size anatomical data: " + str(sizeVMR));
    BrainVoyager.PrintToLog("Marked voxels: " + str(sizeMarkedVMR));
    sizeMarkedFMR = np.sum(t2Data1D)
    BrainVoyager.PrintToLog("Size functional data: " + str(sizeFMR));
    BrainVoyager.PrintToLog("Marked voxels: " + str(sizeMarkedFMR));

    #dice = np.sum(t2Data1D[t1Data1D==marked])*2.0 / (np.sum(t2Data1D) + np.sum(t1Data1D)) # set version
    dice = (2 * np.sum( np.dot(t1Data1D,t2Data1D) )) / (sizeMarkedVMR + sizeMarkedFMR)# vector version
    BrainVoyager.PrintToLog("Sørensen-Dice coefficient: " + str(round(dice, 4))) 

docvmr.Close()
docfmr.Close()



